---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_pc.vhd]
--	Easy 1 Program Counter
--	This module describes the Easy 1 Program Counter.
--	The Easy 1 program counter is composed of a 
--	16 bits register that holds the address for the
--	next instruction, two multiplexers and an adder.
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee;
use ieee.std_logic_1164.all; 
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all; 
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@clock		- clock source
--	@reset    	- reset signal
--	@pc_is     	- selection signal for the first 
--				  mux. This signal selects the source
--				  to increase by two; the A_BUS or PC.
--	@pc_sel		- selection signal for the second 
--				  mux. This signal selects the source
--			 	  for the PC register.
--	@pc_le    	- signal to load the register with the 
--				  next instruction address.
--	@a_bus     	- A Bus
--	@pc_out		- output of the program counter
--					  
entity easy1_pc is
	port (
		clock		: in std_logic:='0';	 
		reset		: in std_logic:='0';
		pc_is       : in  std_logic:='0';                      -- pc is bit
		pc_sel      : in  std_logic_vector(1 downto 0):="00";   -- pc select	 
		pc_le       : in  std_logic:='0';                       -- pc select
		a_bus       : in  std_logic_vector(15 downto 0):=X"0000";  -- address bus
		pc_out      : out std_logic_vector(15 downto 0):=X"0000"); -- pc out 
end easy1_pc;	 
--
----------------------------------------------------
--	Architecture declaration
--	Signals :
--		@pc_reg			- signal that connects the second
--					  	  multiplexer with the register.
--		@pc_out2		- signal that connects the register
--						  output to the PC output. 
--
--	Components :
--		@easy1_reg_ms	- master/slave register
--
architecture pc_arq  of easy1_pc is

component easy1_reg_ms
		port(
			clock	:	in std_logic:='0';
			input	:	in std_logic_vector(15 downto 0):=X"0000";
			load	:	in std_logic:='0';
			clear	:	in std_logic:='0';
			o		:	out std_logic_vector(15 downto 0):=X"0000"
			);
	end component;
															   
	signal pc_reg   : std_logic_vector (15 downto 0):=X"0000";
	signal pc_out2  : std_logic_vector (15 downto 0):=X"0000";
	
begin
	--	Port mapping:
	--		Component instances are mapped to internal signals,
	--		inputs, and outputs.
	register_pc : easy1_reg_ms
	port map (     			  
		clock	   => clock,
		input      => pc_reg,
		load   	   => pc_le,
		clear      => reset,
		o          => pc_out2);  
	
	--	Sensitivity List :
	--		This list holds the signals that modify and 
	--		control the register data. This register is
	--		sensitive to the clock source, the load signal,
	--		the clear signal and the input data.
	
	mux_pcsel: process (pc_le,clock,pc_is,pc_sel,a_bus,reset)
	begin
		-- Multiplexer connected to the input of the register.
		case pc_sel is 
			--	Input to the register comes from the A bus
			when "00" =>
			pc_reg<=a_bus after DELAY_1_LEVEL; 	  
			--	Initialize the register to low (X"0000")
			when "01" =>
			pc_reg<= X"0000" after DELAY_1_LEVEL; 
			--	Input to the register comes from the adder.
			--	Address + 2.
			when "10" =>
			--	Multiplexer connected to the input of the adder.
			case pc_is is  
				--	Input to the adder comes from the A bus.
				when '0' =>
				pc_reg <= a_bus + X"0002" after DELAY_1_LEVEL;
				--	Input to the adder comes from the PC register.
				when '1' =>
				pc_reg <= pc_out2 + X"0002" after DELAY_1_LEVEL;	  
				when others => null;
			end case;	   					   
			when others => null;
		end case;
	end process; 
	--	Connect the output of the register to the output of the PC.
	pc_out<=pc_out2 after DELAY_1_LEVEL;	
	
end pc_arq ;
----------------------------------------------------
----------------------------------------------------