---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_processor.vhd]
--	Easy 1 Processor
--	This module describes the Easy 1 Processor.
--	The Easy 1 processor contains the data paths
--	and the control unit. This entity contains input
--	and output for memory interface.
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee ;
use ieee.std_logic_1164.all; 
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@address_bus	- memory address Bus
--	@data_in	   	- memory data input Bus
--	@data_out		- memory data ouput Bus
--	@control_bus	- memory control signal
--	@reset     		- reset signal
--	@clock  	   	- clock source
--
entity easy1_processor is
	
	port (
		address_bus : out std_logic_vector(15 downto 0):=X"0000";
		data_in   	: in std_logic_vector(15 downto 0):=X"0000";
		data_out	: out std_logic_vector(15 downto 0):=X"0000";
		control_bus : out std_logic_vector(1 downto 0):="00";
		clock		: in std_logic:='0';
		reset       : in std_logic:='0';
		-- OUTPUT SIGNALS -- 
		AC			: out std_logic_vector(15 downto 0):=X"0000";
		PC			: out std_logic_vector(15 downto 0):=X"0000";
		A0			: out std_logic_vector(15 downto 0):=X"0000";
		DI			: out std_logic_vector(15 downto 0):=X"0000";
		IR			: out std_logic_vector(15 downto 0):=X"0000";
		ABUS		: out std_logic_vector(15 downto 0):=X"0000"
		-- OUTPUT SIGNALS -- 
		);
	
end easy1_processor;
--
----------------------------------------------------
--	Architecture declaration
--	Signals :
--		@tmp_alu_op		- ALU opcode
--		@tmp_pc_sel		- control signal for the PC
--		@tmp_ir_le		- load signal for the IR
--		@tmp_pc_is		- control signal for the PC
--		@tmp_pc_le		- load signal for the PC
--		@tmp_di_le		- load signal for the DI
--		@tmp_ac_le		- load signal for the AC
--		@tmp_a0_sel		- control signal for the A0
--		@tmp_a0_le		- load signal for the A0
--		@tmp_sxt_sel	- control signal for the SXTL
--		@tmp_opcode		- instruction opcode
--		@tmp_i			- indirect bit
--		@tmp_ac_15		- sign bit
--		@tmp_c_state	- current state
--		@tmp_n_state	- next state
--
--	Components :
--		@easy1_cu			- Easy 1 Control Unit
--		@easy1_data_paths 	- Easy 1 Data Paths
--		@easy1_a0 			- Easy 1 Address Register
--		@easy1_reg_st		- Easy 1 State Register
--
architecture arq of easy1_processor is 
	
	component easy1_cu
		port( 
			clock 	 	: in  std_logic:='0';
			reset       : in  std_logic:='0';
			alu_op      : out std_logic_vector(2 downto 0):="000";
			pc_sel      : out std_logic_vector(1 downto 0):="00";
			ir_le       : out std_logic:='0';
			pc_is       : out std_logic:='0';
			pc_le	 	: out std_logic:='0';
			di_le       : out std_logic:='0';
			ac_le       : out std_logic:='0';
			a0_sel      : out std_logic:='0';
			a0_le       : out std_logic:='0';
			sxt_sel     : out std_logic_vector(1 downto 0):="00";
			next_state  : out std_logic_vector(3 downto 0):="0000";
			control_bus : out std_logic_vector(1 downto 0):="00";
			curr_state  : in  std_logic_vector(3 downto 0):="0000";
			opcode      : in  std_logic_vector(4 downto 0):="00000";
			i           : in  std_logic:='0';
			ac_15       : in  std_logic:='0'
			);	
	end component;
	
	component easy1_data_paths
		port(
			reset 	 	: in std_logic:='0';
			clock 	 	: in  std_logic:='0';
			alu_op      : in std_logic_vector(2 downto 0):="000";
			pc_sel      : in std_logic_vector(1 downto 0):="00";
			ir_le       : in std_logic:='0';
			pc_is       : in std_logic:='0';
			pc_le	 	: in std_logic:='0';
			di_le       : in std_logic:='0';
			ac_le       : in std_logic:='0';
			ac_cl		: in std_logic:='0';
			a0_sel      : in std_logic:='0';
			a0_le       : in std_logic:='0';
			sxt_sel     : in std_logic_vector(1 downto 0):="00";
			opcode      : out std_logic_vector(4 downto 0):="00000";
			i           : out std_logic:='0';
			ac_15       : out std_logic:='0';
			data_in		: in std_logic_vector(15 downto 0):=X"0000";
			data_out	: out std_logic_vector(15 downto 0):=X"0000";
			address_bus	: out std_logic_vector(15 downto 0):=X"0000";
			-- OUTPUT SIGNALS -- 
			AC			: out std_logic_vector(15 downto 0):=X"0000";
			PC			: out std_logic_vector(15 downto 0):=X"0000";
			A0			: out std_logic_vector(15 downto 0):=X"0000";
			DI			: out std_logic_vector(15 downto 0):=X"0000";
			IR			: out std_logic_vector(15 downto 0):=X"0000";
			ABUS		: out std_logic_vector(15 downto 0):=X"0000"
			-- OUTPUT SIGNALS -- 
			);
	end component;
	
	component easy1_reg_st
		port(				  
			i		:	in std_logic_vector(3 downto 0):="0000";
			load	:	in std_logic:='0';
			clear	:	in std_logic:='0';
			o		:	out std_logic_vector(3 downto 0):="0000"
			);
	end component;
	
	signal tmp_alu_op	: std_logic_vector(2 downto 0):="000";
	signal tmp_pc_sel	: std_logic_vector(1 downto 0):="00";
	signal tmp_ir_le	: std_logic:='0';
	signal tmp_pc_is	: std_logic:='0';
	signal tmp_pc_le	: std_logic:='0';
	signal tmp_di_le	: std_logic:='0';
	signal tmp_ac_le	: std_logic:='0';
	signal tmp_a0_sel	: std_logic:='0';
	signal tmp_a0_le	: std_logic:='0';
	signal tmp_sxt_sel	: std_logic_vector(1 downto 0):="00";
	signal tmp_opcode	: std_logic_vector(4 downto 0):="00000";
	signal tmp_i		: std_logic:='0';
	signal tmp_ac_15	: std_logic:='0';
	signal tmp_c_state	: std_logic_vector(3 downto 0):="0000";	
	signal tmp_n_state	: std_logic_vector(3 downto 0):="0000";	
	
begin	   
	--	Port mapping:
	--		Component instances are mapped to internal signals,
	--		inputs, and outputs.
	--
	--	Control Unit
	cu: easy1_cu
	port map(
		clock => clock,
		reset => reset, 
		alu_op => tmp_alu_op,
		pc_sel => tmp_pc_sel,
		ir_le => tmp_ir_le,
		pc_is => tmp_pc_is,
		pc_le => tmp_pc_le,
		di_le => tmp_di_le,
		ac_le => tmp_ac_le,
		a0_sel => tmp_a0_sel,
		a0_le => tmp_a0_le,
		sxt_sel => tmp_sxt_sel,
		next_state => tmp_n_state,
		control_bus => control_bus,
		curr_state => tmp_c_state,
		opcode => tmp_opcode,
		i => tmp_i,
		ac_15 => tmp_ac_15
		);			
		
	--	State Register
	--	This latch holds the processor state value.
	--	There is no need to use a master/slave register;
	--	instead a latch is used since a ms register causes
	--	an extra period delay for the next state to be 
	--	accesible for the control unit.
	STATE_REG: easy1_reg_st
	port map(
		i => tmp_n_state,
		load => clock,
		clear => reset,
		o => tmp_c_state
		);
		
	--	Data Paths
	data_paths: easy1_data_paths
	port map(
		clock => clock,
		reset=> reset,
		alu_op => tmp_alu_op,
		pc_sel => tmp_pc_sel,
		ir_le => tmp_ir_le, 
		pc_is => tmp_pc_is,
		pc_le => tmp_pc_le,
		di_le => tmp_di_le,
		ac_le => tmp_ac_le,
		ac_cl => reset,
		a0_sel => tmp_a0_sel,
		a0_le => tmp_a0_le,
		sxt_sel => tmp_sxt_sel,
		opcode => tmp_opcode,
		i => tmp_i,
		ac_15 => tmp_ac_15,
		data_in => data_in,
		data_out => data_out,
		address_bus => address_bus,
		-- OUTPUT SIGNALS --
		AC => AC,
		PC => PC,
		A0 => A0,
		DI => DI,
		IR => IR,
		ABUS => ABUS
		-- OUTPUT SIGNALS --
		);
	
end arq;
----------------------------------------------------
----------------------------------------------------