---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_RAM.vhd]
--	Easy 1 Memory Module
--	Behavioral description of a memory module.
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_bit.all;		 
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@clock  	- clock source
--	@mem_op 	- memory operation
--	@Addr		- memory address from Address Bus
--	@data_in	- input data from Data Bus
--	@data_out	- output data to Data Bus
--												   
entity easy1_RAM is
	port(	
		clock		:	in std_logic:='0';	
		mem_op		:	in std_logic_vector(1 downto 0):="00";
		Addr		:	in std_logic_vector(15 downto 0):=X"0000";
		data_in		: 	in std_logic_vector(15 downto 0):=X"0000";
		data_out	:	out std_logic_vector(15 downto 0):=X"0000" 
		);
end easy1_RAM;
--
----------------------------------------------------
--	Architecture declaration
--	Types :
--		@ram_array		- 1Kb array of 8 bit bytes
--
--	Signals :
--		@tmp_ram		- temporary memory array.
--						  This is the memory used
--						  in the simulation.
--		@program		- memory array that holds 
--						  the program. This segment
--						  of memory is copied to
--						  memory when the simulation 
--						  begins.
--
architecture behav of easy1_RAM is
	type ram_array is array (0 to 1023) of 
	std_logic_vector(7 downto 0); 
	
	signal tmp_ram: ram_array := 
	(OTHERS => "ZZZZZZZZ");	   
	
	signal program : ram_array
	:=(								
	------------------------------------------------
	------------------------------------------------
	-- 		Program Code Goes Here				  --
    ------------------------------------------------ 
	------------------------------------------------
		"00011000",		-- And 		0
		"00000000",					   
		"00011100",		-- Add 		3  > 3
		"00000011",
--		"00001000",		-- BrN 		6
--		"00000110",
		"00011011",		-- NOP
		"11111111",					   
		"00011111",		-- Add 		-6 > -3
		"11111010",
		"00011011",		-- NOP
		"11111111",					   
		"00011100",		-- Add 		3 > 0
		"00000011",
		"00011011",		-- NOP
		"11111111",					   
		"00011111",		-- Add 		-1 >  -1
		"11111111",
		"00011100",		-- Add 		3  >  2
		"00000011",
--		"00010000",		-- Store 	100
--		"01100100",
--		"00000000",		-- Comp
--		"00000000",	  
--		"00000100",		-- Shr
--		"00000000",
--		"00010000",		-- Store 	102
--		"01100110",
--		"00010100",		-- Load 	100
--		"01100100",
--		"00010100",		-- Load 	102
--		"01100110",
--		"00001100",		-- Jump 	200
--		"11001000",
		---------------------------------------
		--"00011000",	 	-- And 		0
		--"00000000",
		--"00011100",		-- Add		102
		--"01100110",
		--"00010000",		-- Store 	100
		--"01100100",
		--"00011000",	 	-- And 		0
		--"00000000",
		--"00011100",		-- Add		5
		--"00000101",
		--"00010000",		-- Store 	102
		--"01100110",
		--"00011100",		-- Add		10
		--"00001010",
		--"10010100",		-- Loadi 	100
		--"01100100",
		--"10010000",		-- Storei 	100
		--"01100100",
		--"00011000",	 	-- And 		0
		--"00000000",
		--"00010100",		-- Load 	102
		--"01100110",
		--"00010000",		-- Store	100
		--"01100100",
		--"10011000",		-- Andi		102
		--"01100110",
		--"10011100",		-- Addi		102
		--"01100110",
		--"10001100",		-- Jumpi	100
		--"01100100",					
		--"00011100",	 	-- And 		0
		--"00000000",
		--"00000000",		-- Comp 	0	 	
		--"00000000",
		--"10001000",		-- BrNi		100
		--"01100100",		 
	------------------------------------------------
	------------------------------------------------
	-- 		Program Code Ends Here				  --
    ------------------------------------------------ 
	------------------------------------------------
		others => X"FF"
	);

begin 
	
	--	Procedures :
	--		@load		- verifies there is enough memory
	--					  to load the program and loads
	--					  the program into memory.
	--		@do_write	- write data from data_in into
	--					  the corresponding address.
	--		@do_read 	- read data from the corresponding
	--					  address into data_out.
	process is				   
		procedure load is
		begin
			assert program'length <= tmp_ram'length
			report "MEMORY OUT OF SPACE"
			severity failure;
			tmp_ram(program'range)<=program;
		end load;
		
		procedure do_write is
		begin
			tmp_ram(conv_integer(Addr)) <= data_in(15 downto 8);
			tmp_ram(conv_integer(Addr+1)) <= data_in(7 downto 0);
		end do_write;
		
		procedure do_read is
		begin
			data_out <= tmp_ram(conv_integer(Addr))&tmp_ram(conv_integer(Addr+1));
		end do_read;
		
	begin
		--	Load the program into memory.
		load;
		--	Wait until an operation is requested.
		loop
			wait on mem_op until rising_edge(clock) or mem_op'event;
			case mem_op is											
				--	No Operation (NOP)
				when "00" => null;	  
				--	Read Operation (RD)
				when "01" =>
				do_read;		 
				--	Write Operation (WR)
				when "10" =>
					do_write;
				when others => null;
			end case;
		end loop;
	end process;
end behav;
---------------------------------------------------
---------------------------------------------------